@@ -20,9 +20,10 @@
 */
 
 #include <algorithm>
-#include <iostream>
-#include <set>
 #include <getopt.h>
+#include <iostream>
+#include <string>
+#include <vector>
 
 #include <osmium/io/any_input.hpp>
 #include <osmium/io/input_iterator.hpp>
@@ -30,6 +31,8 @@
 #include <osmium/io/pbf_output.hpp>
 #include <osmium/handler.hpp>
 #include <osmium/osm/entity_bits.hpp>
+#include <osmium/util/memory.hpp>
+#include <osmium/util/verbose_output.hpp>
 
 #include "return_codes.hpp"
 
@@ -38,22 +41,25 @@ void print_help() {
               << "\nOptions:\n"
               << "  -h, --help           - This help message\n"
               << "  -o, --output=OSMFILE - Where to write output (default: none)\n"
+              << "  -v, --verbose        - Verbose output\n"
               << "  -V, --version        - Show version and exit\n"
               << "\n";
 }
 
 int main(int argc, char* argv[]) {
     std::string output_filename;
+    bool verbose = false;
 
     static struct option long_options[] = {
         {"help",         no_argument, 0, 'h'},
         {"output", required_argument, 0, 'o'},
+        {"verbose",      no_argument, 0, 'v'},
         {"version",      no_argument, 0, 'V'},
         {0, 0, 0, 0}
     };
 
     while (1) {
-        int c = getopt_long(argc, argv, "ho:V", long_options, 0);
+        int c = getopt_long(argc, argv, "ho:vV", long_options, 0);
         if (c == -1)
             break;
 
@@ -64,6 +70,9 @@ int main(int argc, char* argv[]) {
             case 'o':
                 output_filename = optarg;
                 break;
+            case 'v':
+                verbose = true;
+                break;
             case 'V':
                 std::cout << "osmcoastline_filter version " OSMCOASTLINE_VERSION "\n"
                           << "Copyright (C) 2012-2016  Jochen Topf <jochen@topf.org>\n"
@@ -76,6 +85,11 @@ int main(int argc, char* argv[]) {
         }
     }
 
+    // The vout object is an output stream we can write to instead of
+    // std::cerr. Nothing is written if we are not in verbose mode.
+    // The running time will be prepended to output lines.
+    osmium::util::VerboseOutput vout(verbose);
+
     if (output_filename.empty()) {
         std::cerr << "Missing -o/--output=OSMFILE option\n";
         exit(return_code_cmdline);
@@ -98,6 +112,7 @@ int main(int argc, char* argv[]) {
 
         std::vector<osmium::object_id_type> ids;
 
+        vout << "Reading ways (1st pass through input file)...\n";
         {
             osmium::io::Reader reader(infile, osmium::osm_entity_bits::way);
             auto ways = osmium::io::make_input_iterator_range<const osmium::Way>(reader);
@@ -113,17 +128,32 @@ int main(int argc, char* argv[]) {
             reader.close();
         }
 
+        vout << "Preparing node ID list...\n";
         std::sort(ids.begin(), ids.end());
         auto last = std::unique(ids.begin(), ids.end());
 
+        vout << "Reading nodes (2nd pass through input file)...\n";
         {
             osmium::io::Reader reader(infile, osmium::osm_entity_bits::node);
             auto nodes = osmium::io::make_input_iterator_range<const osmium::Node>(reader);
-            std::copy_if(nodes.cbegin(), nodes.cend(), output_it, [&ids, &last](const osmium::Node& node){
+
+            auto first = ids.begin();
+            std::copy_if(nodes.cbegin(), nodes.cend(), output_it, [&first, &last](const osmium::Node& node){
+                while (*first < node.id() && first != last) {
+                    ++first;
+                }
+
+                if (node.id() == *first) {
+                    if (first != last) {
+                        ++first;
+                    }
+                    return true;
+                }
+
                 const char* natural = node.get_value_by_key("natural");
-                return std::binary_search(ids.begin(), last, node.id()) ||
-                       (natural && !strcmp(natural, "coastline"));
+                return natural && !strcmp(natural, "coastline");
             });
+
             reader.close();
         }
 
@@ -132,5 +162,12 @@ int main(int argc, char* argv[]) {
         std::cerr << "io error: " << e.what() << "'\n";
         exit(return_code_fatal);
     }
+
+    vout << "All done.\n";
+    osmium::MemoryUsage mem;
+    if (mem.current() > 0) {
+        vout << "Memory used: current: " << mem.current() << " MBytes\n"
+             << "             peak:    " << mem.peak() << " MBytes\n";
+    }
 }
 
